@@ -5,10 +5,9 @@
 		*/
 
 #include "config.h"
-
 #include <unistd.h>
 #include <sys/types.h>
-#ifdef REMOTE
+#ifdef SUPPORT_REMOTE
 #include <netdb.h>
 #endif
 #include <sys/ioctl.h>
@@ -28,13 +27,14 @@ report_error(char *err_str1, char *err_str2, int fatal)
 {
   int console_fd;
 
-  if ((console_fd=open("/dev/console", O_RDWR))<0)
+  if ((console_fd=open(_PATH_CONSOLE, O_RDWR))<0)
     {
       /* fall back to stderr */
       console_fd=2;
     }
 
-  write(console_fd, "login: ", 7);
+  write(console_fd, PROGRAM_NAME, sizeof(PROGRAM_NAME));
+  write(console_fd, ": ", 2);
   write(console_fd, err_str1, strlen(err_str1));
   write(console_fd, err_str2, strlen(err_str2));
   write(console_fd, "\n", 1);
@@ -45,30 +45,21 @@ report_error(char *err_str1, char *err_str2, int fatal)
     exit(1);
 }
 
-int
+void
 parse_commandline(int argc, char **argv, struct settings *settings)
 {
   int curr_arg;
 
-  settings->have_username=0;
-#ifndef VERY_SMALL
-  settings->force=0;
-#ifdef REMOTE
-  settings->remote_login=0;
-#endif
-#endif
-
   if (argc==1)
-    return(1);
+    return;
 
   if (argc==2)
     {
       strcpy(settings->username,argv[1]);
       settings->have_username=1;
-      return(1);
+      return;
     }
 
-#ifndef VERY_SMALL
   if (argc>2)
     {
       curr_arg=1;
@@ -80,12 +71,18 @@ parse_commandline(int argc, char **argv, struct settings *settings)
 #ifdef REMOTE
 	      case 'h': /* "-h" == hostname */
 		curr_arg++;
+		if (curr_arg>=argc)
+		  report_error("option -h requires an additional argument:",
+			       " -h hostname", 1);
 		strcpy(settings->hostname, argv[curr_arg]);
 	        settings->remote_login=1;
 		break;
 #endif
 	      case 'f': /* "-f" == pre-autenticated login (force) */
 		curr_arg++;
+		if (curr_arg>=argc)
+		  report_error("option -f requires an additional argument:",
+			       " -f username", 1);
 		strcpy(settings->username, argv[curr_arg]);
 		settings->have_username=1;
 		settings->force=1;
@@ -94,9 +91,6 @@ parse_commandline(int argc, char **argv, struct settings *settings)
 	  curr_arg++;
 	}
     }
-#endif
-
-  return(1);
 }
 
 int
@@ -104,15 +98,16 @@ get_username(struct settings *settings)
 {
   int username_length;
 
-  write(1, LOGIN_PROMPT, LOGIN_PROMPT_LENGTH);
+  write(1, LOGIN_PROMPT, sizeof(LOGIN_PROMPT));
 
   if ((username_length=read(0, settings->username, MAX_USERNAME_LENGTH))<0)
     exit(1);
 
-  if (username_length==0) /* They hit ^D */
+  /* If the user presses ^D at the login: prompt, we should
+     report no error. */
+  if (username_length==0)
     exit(0);
 
-
   /* remove the newline character */
   settings->username[username_length-1]='\0';
 
@@ -129,15 +124,16 @@ get_password(struct settings *settings)
   ioctl(0, TCGETS, &old_termios);
   memcpy((void *) &new_termios, (void *) &old_termios, sizeof(struct termios));
 
+  /* turn off echo */
   new_termios.c_lflag&=~(ECHO|ISIG);
 
   ioctl(0, TCSETS, &new_termios);
 
-  write(1, PW_PROMPT, PW_PROMPT_LENGTH);
+  write(1, PW_PROMPT, sizeof(PW_PROMPT));
 
   pw_length=read(0, settings->password, MAX_PW_LENGTH);
   if (pw_length<0)
-      exit(1);
+      report_error("read returned an error", "", 1);
 
   /* remove the newline character */
   settings->password[pw_length-1]='\0';
@@ -153,6 +149,10 @@ int
 verify_password(struct settings settings, char *password)
 {
   char salt[3];
+
+  /* The salt is the first two charcters of the password entry
+     in the passwd file. */
+
   *salt=*password;
   *(salt+1)=*(password+1);
   *(salt+2)='\0';
@@ -194,6 +194,9 @@ update_utmp(struct settings settings)
   memcpy(utmp_entry.ut_user, settings.username, sizeof(utmp_entry.ut_user));
   memcpy(utmp_entry.ut_line, ttyname(0)+5, sizeof(utmp_entry.ut_line));
   memcpy(utmp_entry.ut_id, ttyname(0)+strlen(ttyname(0))-2, sizeof(utmp_entry.ut_id));
+
+  /* We were previously a LOGIN process.. but we're about to become a
+     USER process (once the shell is executed). */
   utmp_entry.ut_type=USER_PROCESS;
 
 #ifdef REMOTE
@@ -248,32 +251,30 @@ display_motd(void)
   close(motd_fd);
 }
 
-int
+void
 login_failure(struct settings *settings, int *num_failures)
 {
+
   (*num_failures)++;
+
   if (*num_failures==MAX_FAILURES)
 #ifdef REMOTE
     if (settings->remote_login)
-      {
 	report_error("REPEATED LOGIN FAILURES FROM ", settings->hostname, 1);
-      }
     else
 #endif
       report_error("REPEATED LOGIN FAILURES ON ", ttyname(0), 1);
 
-  write(1, FAIL_MESSAGE, FAIL_MESSAGE_LENGTH);
+  write(1, FAIL_MESSAGE, sizeof(FAIL_MESSAGE));
     
   settings->have_username=0;
-  
-  return(1);
+
 }
 
 void
 check_nologin(void)
 /* Check to see if there is a nologin file and if so, display
    it and exit. */
-
 {
   int nologin_fd;
   int bytes_read;
@@ -286,6 +287,50 @@ check_nologin(void)
     write(1, buff, bytes_read);
 
   close(nologin_fd);
+
+  exit(1);
+}
+
+void
+log_in(struct settings settings, struct passwd *passwd_entry)
+{
+  char shell_name[512];
+
+  /* set the tty permissions */
+  fchown(0, passwd_entry->pw_uid, passwd_entry->pw_gid);
+  fchmod(0, TTY_MODE);
+
+  /* Set the user's group ID */
+  setgid(passwd_entry->pw_gid);
+
+  /* Initialize the user's secondary groups.  initgroups() needs
+     to be run as root. */
+  initgroups(passwd_entry->pw_name, passwd_entry->pw_gid);
+
+  /* Change to the user's UID. */
+  setuid(passwd_entry->pw_uid);
+
+  /* change to the home directory, prepare to exec the shell */
+  if (chdir(passwd_entry->pw_dir)<0)
+      report_error(passwd_entry->pw_dir, " does not exist!", 1);
+
+  /* set the appropriate environment variables */
+  setenv("HOME", passwd_entry->pw_dir, 1);
+  setenv("SHELL", passwd_entry->pw_shell, 1);
+  setenv("PATH", _PATH_DEFPATH, 0);
+
+  display_motd();
+
+  /* We must set the first character of argv[0] to - so the shell
+     knows that it is a login shell */
+  *shell_name='-';
+  shell_name[1]='\0';
+  execl(passwd_entry->pw_shell, strcat(shell_name, passwd_entry->pw_shell),
+	NULL);
+
+  report_error("no shell: ", passwd_entry->pw_shell, 1);
+
+  exit(1);
 }
 
 int
@@ -294,32 +339,27 @@ main(int argc, char **argv)
   struct settings settings;
   struct passwd *passwd_entry=NULL;
   int verified;
-  int curr_arg;
   int num_failures;
-  char shell_name[512];
+
+  /* Clear the settings structure */
+  settings.force=DEFAULT_FORCE;
+  settings.have_username=DEFAULT_HAVE_USERNAME;
+#ifdef SUPPORT_REMOTE
+  settings.remote_login=DEFAULT_REMOTE_LOGIN;
+#endif
 
   parse_commandline(argc, argv, &settings);
 
   check_nologin();
 
-  /* It seems like a security hole to keep the args to login visible
-     in a ps (what if someone accidentally typed his password instead of
-     his login?), so clear them now that we're done with them. */
-  for (curr_arg=1;curr_arg<argc-1;curr_arg++)
-    memset(argv[curr_arg], 0, strlen(argv[curr_arg]));
-
 #ifdef REMOTE
   if (settings.remote_login)
     get_hostent(&settings);
 #endif
 
- /* Now begin the read-login, read-password, verify loop */
-#ifndef VERY_SMALL
+ /* Now begin the read login->read password->verify loop */
   verified=settings.force;
-#else
-  verified=0;
-#endif
-num_failures=0;
+  num_failures=0;
   while (!verified)
     {
 
@@ -335,47 +375,19 @@ num_failures=0;
       if (passwd_entry!=NULL)
 	  verified=verify_password(settings, passwd_entry->pw_passwd);
 
+      /* clear the password entry - paranoia */
       memset(settings.password, 0, MAX_PW_LENGTH);
 
       if (!verified)
 	  login_failure(&settings, &num_failures);
-
     }
+
 #ifndef NO_UTMP
   /* Write the utmp and the wtmp */
   update_utmp(settings);
 #endif
 
-  /* set the tty permissions */
-  fchown(0, passwd_entry->pw_uid, passwd_entry->pw_gid);
-  fchmod(0, 0620);
-
-  setgid(passwd_entry->pw_gid);
-
-  /* initgroups needs to be called by root... */
-  initgroups(passwd_entry->pw_name, passwd_entry->pw_gid);
-
-  setuid(passwd_entry->pw_uid);
-
-
-  /* change to the home directory, prepare to exec the shell */
-  if (chdir(passwd_entry->pw_dir)<0)
-      report_error(passwd_entry->pw_dir, " does not exist!", 1);
-
-  /* set the appropriate environment variables */
-  setenv("HOME", passwd_entry->pw_dir, 1);
-  setenv("SHELL", passwd_entry->pw_shell, 1);
-  setenv("PATH", _PATH_DEFPATH, 0);
-
-  display_motd();
-
-  /* We must set the first character of argv[0] to - so the shell
-     knows that it is a login shell */
-  strcpy(shell_name, "-");
-  execl(passwd_entry->pw_shell, strcat(shell_name, passwd_entry->pw_shell),
-	NULL);
-
-  report_error("no shell: ", passwd_entry->pw_shell, 1);
+  log_in(settings, passwd_entry);
   
   return(1);
 }
